#pragma once

#include "feature.h"

#include <vector>
using namespace std;

const int NUM_EDGES = 4;
enum EDGE_TYPE { FARM, ROAD, CITY, RIVER };
const string EdgeType[] = { "farm", "road", "city", "river" };

inline EDGE_TYPE ReadEdge(string str)
{
	for(int i=0;i<NUM_EDGES;i++)
		if(str == EdgeType[i])
			return (EDGE_TYPE) i;

	return FARM;
}

class Tile;

class Edge
{
public:
	Edge(Tile *pParent);
	~Edge();

	void Display();

	Tile *GetParent() const { return m_pParent; }
	bool HasFeature(Feature *pFeature);

	Edge *GetNeighbor() const { return m_pNeighbor; }
	Tile *GetNeighborTile() const { return (m_pNeighbor ? m_pNeighbor->GetParent() : 0); }
	EDGE_TYPE GetType() const { return m_type; }
	Feature *GetFeature(int i) const { return m_features[i]; }

	void SetNeighbor(Edge *pNeighbor) { m_pNeighbor = pNeighbor; }
	void SetType(EDGE_TYPE type) { m_type = type; }
	void SetType(string type) { m_type = ReadEdge(type); }
	void SetFeature(Feature *pFeature, int i) { m_features[i] = pFeature; }
	void SetFeatures(string data, vector <Feature *>& features);

	vector <Feature *> AttachTo(Edge *pEdge);
	void Replace(Feature *pOut, Feature *pIn, bool doParent = true);

private:
	Tile *m_pParent;
	Edge *m_pNeighbor;
	EDGE_TYPE m_type;
	Feature *m_features[3];
};
