#include "game.h"

#include <iostream>
#include <fstream>
using namespace std;

//const int sX = 64, sY = 64;
const int jX = 800, jY = 600;
const int jsX = 128, jsY = 128;

Game::Game(): m_offset(100, 100), m_tileSize(64, 64), m_zoom(1), m_zoomSpeed(.9)
{
	m_pTile = 0;
	m_pJustPlaced = 0;
	m_tilesPlaced = false;
}

Game::~Game()
{
	for(unsigned i=0;i<m_features.size();i++)
		delete m_features[i];
	for(TileIt i=m_tiles.begin();i!=m_tiles.end();i++)
		delete i->second;

	delete m_pTile;
}

void Game::Initialize()
{
	ifstream fin("Data/tiles.txt");

	string str;
	while(1) {
		fin >> str;
		if(fin.eof())
			break;

		m_tileNames.push_back(str);
	}

	fin.close();
	
	m_player.SetupMeeples();
}

void Game::Draw()
{
	CreateOrthoWindowf(0, 0, 1, 1);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	glTranslatef(m_pSurface->w / 2, m_pSurface->h / 2, 0);

	for(TileIt i=m_tiles.begin();i!=m_tiles.end();i++)
		i->second->Draw(m_offset.m_x * m_zoom, m_offset.m_y * m_zoom, m_tileSize.m_x * m_zoom, m_tileSize.m_y * m_zoom);

	glTranslatef(-m_pSurface->w / 2, -m_pSurface->h / 2, 0);

	if(m_pTile) {
		int x, y;
		m_pInput->GetMouseCoords(x, y);
		m_pTile->Draw(x - .5 * m_tileSize.m_x * m_zoom, y - .5 * m_tileSize.m_y * m_zoom,
			m_tileSize.m_x * m_zoom, m_tileSize.m_y * m_zoom);
	}

	if(m_pJustPlaced)
		m_pJustPlaced->Draw(jX, jY, jsX, jsY, true, true);
}

void Game::Update(double dt)
{

}

void Game::OnKeyDown(Key& key)
{
	switch(key.keysym.sym) {
		case SDLK_ESCAPE:
			Quit();
			break;
		case SDLK_F1:
			ToggleFullScreen();
			break;
		case 'f':
			cout << "FPS: " << GetFPS() << endl;
			break;
		case 't':
			for(TileIt i=m_tiles.begin();i!=m_tiles.end();i++)
				i->second->Display();
			break;
		case 'a':
			m_pJustPlaced = 0;
		
			static unsigned t = 0;
			if(t >= m_tileNames.size())
				break;

			if(m_pTile) {
				Delete(m_added);
				m_added.clear();
				delete m_pTile;
				m_pTile = 0;
			}

			m_pTile = new Tile;
			m_added =  m_pTile->Create("Data/" + m_tileNames[t]);
			Add(m_added);
			t++;
			break;
		case 'r':
			if(m_pTile)
				m_pTile->RotateCW();
			break;
		case 'e':
			if(m_pTile)
				m_pTile->RotateCCW();
			break;
		case 'i':
			for(unsigned i=0;i<m_features.size();i++)
				m_features[i]->SayInfo();
			break;
		default:
			break;
	}
}

void Game::OnButtonDown(Button& button)
{
	if(button.button == SDL_BUTTON_LEFT) {
//		int	i = Floor((double) (button.x - m_offset.m_x) / sX),
//				j = Floor((double) (button.y - m_offset.m_y) / sY);

		Point p = ScreenToGrid(Point(button.x, button.y));

		if(m_pJustPlaced) {
			double a = button.x - jX, b = button.y - jY;
			if(0 <= a && a <= jsX && 0 <= b && b <= jsY) {
				Meeple *pMeeple = m_player.GetFreeMeeple();
				if(!pMeeple) {
					cout << "No more meeples left!\n";
				} else if(m_pJustPlaced->AddMeeple(pMeeple, Floor(3 * a / jsX), Floor(3 * b / jsY))) {
					cout << "Adding meeple!\n";
					m_pJustPlaced = 0;
				} else
					cout << "Can't add meeple!\n";
			}
		}

		if(m_pTile) {
			if(!CanPlaceTile(p, m_pTile)) {
				cout << "Cannot place tile at " << p.m_x << ", " << p.m_y << endl;
			} else {
				PlaceTile(p, m_pTile);
				m_pJustPlaced = m_pTile;
				m_pTile = 0;
			}
		}
	}
}

void Game::OnWheelDown(Button& button)
{
	// Zooming
	m_zoom *= m_zoomSpeed;
	if(m_zoom < .5)
		m_zoom = .5;
}

void Game::OnWheelUp(Button& button)
{
	// Zooming
	m_zoom /= m_zoomSpeed;
	if(m_zoom > 2)
		m_zoom = 2;
}

void Game::OnMouseMove(Motion& motion)
{
	// Panning
	if(motion.state & SDL_BUTTON_RMASK) {
		m_offset.m_x += motion.xrel;
		m_offset.m_y += motion.yrel;
	}
}

bool Game::CanPlaceTile(Point p, Tile *pTile)
{
	if(m_tiles.count(p))
		return false;

	if(!m_tilesPlaced)
		return true;

	bool hasNeighbors = false;
	bool tentative = false;

	for(int d=0;d<4;d++) {
		Point q = p.GetNeighbor((DIR) d);
		if(!m_tiles.count(q))
			continue;

		hasNeighbors = true;
		if(!pTile->CanAttach(m_tiles[q], (DIR) d, tentative))
			return false;
	}

	if(!hasNeighbors) {
		cout << "Tile must be placed adjecent to another tile.\n";
		return false;
	}

	if(!tentative) {
		cout << "River tiles must be joined at a river.\n";
		return false;
	}

	// River pieces can't point towards another tile that's two squares away
	if(pTile->IsRiver()) {
		for(int i=0;i<4;i++) {
			if(pTile->GetEdge((DIR) i)->GetType() != RIVER)
				continue;

			// Grab the point that's one away
			// We only care if this is empty
			Point q = p.GetNeighbor((DIR) i);
			if(m_tiles.count(q))
				continue;

			// Grab the point that's two away
			// It had better be empty
			Point r = q.GetNeighbor((DIR) i);
			if(m_tiles.count(r)) {
				cout << "River tiles can't point twards another tile that's two squares away.\n";
				return false;
			}
		}
	}

	return true;
}

bool Game::PlaceTile(Point p, Tile *pTile)
{
	if(!CanPlaceTile(p, pTile))
		return false;

	pTile->SetPoint(p);
	m_tiles[p] = pTile;

	for(int d=0;d<4;d++) {
		Point q = p.GetNeighbor((DIR) d);
		if(!m_tiles.count(q))
			continue;

		Attach(m_tiles[p], m_tiles[q], (DIR) d);
	}

	for(int d=0;d<8;d++) {
		Point q = p.GetNeighbor((DIR) d);
		if(!m_tiles.count(q))
			continue;

		m_tiles[p]->AddNeighbor();
		m_tiles[q]->AddNeighbor();
	}

	for(unsigned i=0;i<m_features.size();i++) {
		if(m_features[i]->IsNowCompleted()) {
			cout << "Completing feature: ";
			m_features[i]->SayInfo();
			m_features[i]->Complete();
		}
	}

	m_tilesPlaced = true;

	return true;
}

// Add
// . Adds the given Features to the list
void Game::Add(vector <Feature *> add)
{
	for(unsigned i=0;i<add.size();i++)
		m_features.push_back(add[i]);
}

// Deletes
// . Removes the given Features from the list and
//   then deletes them.
// . WARNING: There may not be multiple instances
//   of the same Feature. Since this function deletes
//   the Features, it would end up deleting the same
//   pointer twice.
void Game::Delete(vector <Feature *> remove)
{
	for(unsigned i=0;i<remove.size();i++) {
		Feature *pFeature = remove[i];
		for(unsigned j=0;j<m_features.size();j++) {
			if(m_features[j] == pFeature) {
				m_features.erase(m_features.begin() + j);
				j--;
			}
		}
		delete pFeature;
	}
}

void Game::Attach(Tile *pTile1, Tile *pTile2, DIR direction)
{
	Delete(pTile1->Attach(pTile2, direction));
}

Point Game::ScreenToGrid(Point p)
{
	double	x = (p.m_x - .5 * m_pSurface->w) / m_zoom - m_offset.m_x,
				y = (p.m_y - .5 * m_pSurface->h) / m_zoom - m_offset.m_y;

	return Point(Floor(x / m_tileSize.m_x), Floor(y / m_tileSize.m_y));
}
