#include "image.h"

bool Image::LoadBitmap(string fileName, FILTER filter)
{
	// Load the bitmap
	SDL_Surface *pSurface = SDL_LoadBMP(fileName.c_str());
	
	if(!pSurface)
		return false;

	// Generate the texture
	glGenTextures(1, &m_texture);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, m_texture);
	
	// Copy the data onto the texture
	if(filter == MIPMAP)
		glTexImage2D(GL_TEXTURE_2D, 0, 3, pSurface->w, pSurface->h, 0, GL_BGR, GL_UNSIGNED_BYTE, pSurface->pixels);
	else
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, pSurface->w, pSurface->h, GL_BGR, GL_UNSIGNED_BYTE, pSurface->pixels);

	// Set up the type of filtering we want
	switch(filter) {
		case NEAREST:
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
			break;
		case LINEAR:
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			break;
		case MIPMAP:
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			break;
	}
	
	glBindTexture(GL_TEXTURE_2D, 0);
	glDisable(GL_TEXTURE_2D);

	// Free the memory
	if(pSurface)
		SDL_FreeSurface(pSurface);
	
	return true;
}

void Image::Bind()
{
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, m_texture);
}

void Image::UnBind()
{
	glBindTexture(GL_TEXTURE_2D, 0);
	glDisable(GL_TEXTURE_2D);
}
