#include "sdlinput.h"
#include "sdlwindow.h"

void Mouse::MoveTo(int x, int y)
{
	m_x = x;
	m_y = y;
}

Keyboard::Keyboard()
{
	for(int i=0;i<255;i++)
		m_keys[i] = false;
}

void Keyboard::KeyDown(SDLKey key)
{
	if(0 <= key && key < 255)
		m_keys[key] = true;
}

void Keyboard::KeyUp(SDLKey key)
{
	if(0 <= key && key < 255)
		m_keys[key] = false;
}

Input::Input(Window *pWindow): m_pWindow(pWindow)
{

}

Input::~Input()
{

}

void Input::MoveMouse(Motion& motion)
{
	m_pWindow->OnMouseMove(motion);
	m_mouse.MoveTo(motion.x, motion.y);
}

void Input::ButtonDown(Button& button)
{
	if(button.button == SDL_BUTTON_WHEELUP) {
		m_pWindow->OnWheelUp(button);
	} else if(button.button == SDL_BUTTON_WHEELDOWN) {
		m_pWindow->OnWheelDown(button);
	} else {
		m_pWindow->OnButtonDown(button);
		m_mouse.m_buttons[button.button] = true;
	}
}

void Input::ButtonUp(Button& button)
{
	m_pWindow->OnButtonUp(button);
	m_mouse.m_buttons[button.button] = false;
}

void Input::KeyDown(Key& key)
{
	m_pWindow->OnKeyDown(key);
	m_keyboard.KeyDown(key.keysym.sym);
}

void Input::KeyUp(Key& key)
{
	m_pWindow->OnKeyUp(key);
	m_keyboard.KeyUp(key.keysym.sym);
}

void Input::GetMouseCoords(int& x, int& y)
{
	x = m_mouse.m_x;
	y = m_mouse.m_y;
}

bool Input::IsModDown(SDLMod mod)
{
	SDLMod modifiers = SDL_GetModState();
	return (modifiers & mod);
}
