///////////////////////////////////////////////////
// Window class
//
// Author: Jesse Beder
// Data: 2/18/2006
//
// This class wraps an SDL window that runs OpenGL.
// It is designed to be derived from. The derived
// class must override three functions:
// . Initialize
//   This is called before the main loop. All
//   initialization that requires SDL to be loaded
//   should be done here.
// . Draw
//   This is called once per frame.
// . Update
//   This is called once per frame
//
// To load the window, simply call Run(...)


#pragma once

#include <GL/gl.h>
#include <GL/glu.h>
#include "SDL.h"
#include "sdlinput.h"

class Input;

class Window
{
public:
	Window();
	virtual ~Window();

	virtual bool Run(int width, int height, int bpp);
	virtual int GetFPS() const { return m_FPS; }

	virtual bool InitSDL(int width, int height, int bpp);
	virtual bool InitGL();
	virtual void Quit();
	virtual void ToggleFullScreen();

	virtual void DoLoop();
	virtual bool HandleEvents();

	virtual bool CreatePerspectiveWindowf(double x, double y, double X, double Y, double far, double perspective = 45);
	virtual bool CreateOrthoWindowf(double x, double y, double X, double Y);

	virtual bool CreatePerspectiveWindowi(int x, int Y, int width, int height, double far, double perspective = 45);
	virtual bool CreateOrthoWindowi(int x, int Y, int width, int height);
	
	virtual void Initialize() = 0;
	virtual void Draw() = 0;
	virtual void Update(double dt) = 0;

	virtual void OnKeyDown(Key& key) {}
	virtual void OnKeyUp(Key& key) {}
	virtual void OnButtonDown(Button& button) {}
	virtual void OnButtonUp(Button& button) {}
	virtual void OnWheelUp(Button& button) {}
	virtual void OnWheelDown(Button& button) {}
	virtual void OnMouseMove(Motion& motion) {}

protected:
	SDL_Surface *m_pSurface;
	Input *m_pInput;
	int m_videoFlags;
	
	bool m_hasFocus, m_isActive;
	int m_ticks;
	int m_FPS;
};
